'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _contextSubscribe = require('../lib/contextSubscribe');

var _bem = require('../lib/bem');

var _state = require('../lib/state');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

// we only use raf for a minor accessibility feature so it is possible to get away with this little "polyfilling"
var raf = typeof window !== 'undefined' && 'requestAnimationFrame' in window ? window.requestAnimationFrame : setTimeout;

var panelInstances = 0;
var panelUniqId = 0;

/*
 * Handles prop integrity and manages the complexity of props vs. context vs. nothing at all
 * @param {object} props
 * @param {object} context
 * @param {string} uniqId
 */
function getTabPanelProps(_ref, _ref2, uniqId) {
    var bem = _ref2.bem,
        tabbordion = _ref2.tabbordion;

    var checkedProp = _ref.checked,
        disabledProp = _ref.disabled,
        idProp = _ref.id,
        index = _ref.index,
        modifiers = _ref.modifiers,
        nameProp = _ref.name,
        type = _ref.type,
        value = _ref.value,
        visibleProp = _ref.visible,
        props = _objectWithoutProperties(_ref, ['checked', 'disabled', 'id', 'index', 'modifiers', 'name', 'type', 'value', 'visible']);

    var _tabbordion$getState = tabbordion.getState(),
        animateContent = _tabbordion$getState.animateContent,
        checkedPanelsRaw = _tabbordion$getState.checkedPanels,
        disabledPanelsRaw = _tabbordion$getState.disabledPanels,
        firstVisiblePanel = _tabbordion$getState.firstVisiblePanel,
        lastVisiblePanel = _tabbordion$getState.lastVisiblePanel,
        panelName = _tabbordion$getState.panelName,
        panelType = _tabbordion$getState.panelType,
        tabbordionId = _tabbordion$getState.tabbordionId;

    var checkedPanels = (0, _state.getArray)(checkedPanelsRaw);
    var disabledPanels = (0, _state.getArray)(disabledPanelsRaw);

    // props have preference over what comes from context
    var name = (nameProp != null ? nameProp : panelName) || tabbordionId || 'tabbordion-panel-' + uniqId;
    var id = idProp || (tabbordionId || name) + '-' + index;

    var checked = checkedProp != null ? checkedProp : checkedPanels.includes(index);
    var disabled = disabledProp != null ? disabledProp : disabledPanels.includes(index);

    var visible = visibleProp === false ? 'hidden' : firstVisiblePanel === index && 'first' || lastVisiblePanel === index && 'last' || 'between';

    var contentId = null;
    var hasContent = null;

    // sniff the id out or use our own (will be exposed via context)
    _react.Children.forEach(props.children, function (child) {
        if (!contentId && child && child.type && child.type.hasContent) {
            hasContent = true;
            contentId = (child.props || child._store && child._store.props || {}).id || null;
        }
    });

    if (!contentId) contentId = id + '-content';

    var _bem$getState = bem.getState(),
        bemModifiers = _bem$getState.bemModifiers,
        bemSeparator = _bem$getState.bemSeparator,
        blockElements = _bem$getState.blockElements;

    return _extends({}, props, {
        animateContent: animateContent,
        bemModifiers: bemModifiers,
        bemSeparator: bemSeparator,
        blockElements: blockElements,
        checked: checked,
        disabled: disabled,
        contentId: contentId,
        hasContent: hasContent,
        id: id,
        index: index,
        name: name,
        type: type != null ? type : panelType,
        value: value != null ? value : String(index),
        visible: visible,
        checkedPanels: checkedPanels,
        disabledPanels: disabledPanels,
        modifiers: bemModifiers ? (0, _state.getArray)(modifiers).concat([bemModifiers[checked ? 'checked' : 'unchecked'], bemModifiers[hasContent ? 'content' : 'noContent'], bemModifiers[disabled ? 'disabled' : 'enabled'], bemModifiers[visible]].concat(_toConsumableArray(firstVisiblePanel === lastVisiblePanel && index === firstVisiblePanel ? [bemModifiers['last']] : []))).concat(animateContent ? [bemModifiers.animated, animateContent] : []) : (0, _state.getArray)(modifiers),
        onChangePanel: tabbordion.onChangePanel
    });
}

var TabPanel = function (_PureComponent) {
    _inherits(TabPanel, _PureComponent);

    function TabPanel(props, context) {
        _classCallCheck(this, TabPanel);

        // essentially an attempt for keeping id and name consistent from page to page even when that data is missing
        var _this = _possibleConstructorReturn(this, (TabPanel.__proto__ || Object.getPrototypeOf(TabPanel)).call(this, props, context));

        panelInstances++;
        // fallback to generate name and id
        _this.uniqId = '' + panelUniqId;
        panelUniqId++;
        // use caching to minimize the need of recalculating stuff all the time
        _this.cachedProps = getTabPanelProps(props, context, _this.uniqId);

        _this.getInputRef = _this.getInputRef.bind(_this);
        _this.onChange = _this.onChange.bind(_this);
        _this.onClickLabel = _this.onClickLabel.bind(_this);

        _this.subscribers = [];

        _this.childContext = {
            tabbordionPanel: {
                getState: function getState() {
                    return {
                        animateContent: _this.cachedProps.animateContent,
                        checked: _this.cachedProps.checked,
                        contentId: _this.cachedProps.contentId,
                        disabled: _this.cachedProps.disabled,
                        inputId: _this.cachedProps.id,
                        index: _this.cachedProps.index,
                        modifiers: _this.cachedProps.modifiers,
                        visible: _this.cachedProps.visible
                    };
                },
                onClickLabel: _this.onClickLabel,
                subscribe: (0, _contextSubscribe.addSubscriber)(_this.subscribers),
                unsubscribe: (0, _contextSubscribe.removeSubscriber)(_this.subscribers)
            }
        };

        _this.panelState = _this.childContext.tabbordionPanel.getState();
        return _this;
    }

    _createClass(TabPanel, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            this.context.bem.subscribe(this);
            this.context.tabbordion.subscribe(this);
        }
    }, {
        key: 'componentWillReceiveProps',
        value: function componentWillReceiveProps(nextProps, nextContext) {
            this.cachedProps = getTabPanelProps(nextProps, nextContext, this.uniqId);

            var panelState = this.childContext.tabbordionPanel.getState();

            if ((0, _state.isShallowlyDifferent)(panelState, this.panelState)) {
                this.subscribers.forEach(function (component) {
                    return component.forceUpdate();
                });
                this.panelState = panelState;
            }
        }
    }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
            this.context.bem.unsubscribe(this);
            this.context.tabbordion.unsubscribe(this);
            delete this.cachedProps;
            panelInstances--;
            if (panelInstances === 0) panelUniqId = 0;
        }
    }, {
        key: 'getChildContext',
        value: function getChildContext() {
            return this.childContext;
        }
    }, {
        key: 'getInputRef',
        value: function getInputRef(input) {
            this.input = input;
        }

        /** Handler for input checkbox or radio */

    }, {
        key: 'onChange',
        value: function onChange(event) {
            if (event.defaultPrevented) {
                return;
            }
            var _cachedProps = this.cachedProps,
                index = _cachedProps.index,
                onChangePanel = _cachedProps.onChangePanel;

            if (onChangePanel) onChangePanel(index);
        }

        /** Handler for TabLabel child */

    }, {
        key: 'onClickLabel',
        value: function onClickLabel() {
            var _this2 = this;

            var _cachedProps2 = this.cachedProps,
                index = _cachedProps2.index,
                onChangePanel = _cachedProps2.onChangePanel,
                type = _cachedProps2.type;

            if (onChangePanel) onChangePanel(index);
            // make sure focus goes to the input element, that is what sane browsers do when a label is clicked
            raf(function () {
                // of course things may go wrong so make sure all expected conditions are met before doing stuff
                if (_this2.input && (_this2.input.checked || type === 'checkbox') && document.activeElement !== _this2.input) {
                    _this2.input.focus();
                }
            });
        }
    }, {
        key: 'render',
        value: function render() {
            var _cachedProps3 = this.cachedProps,
                animateContent = _cachedProps3.animateContent,
                bemModifiers = _cachedProps3.bemModifiers,
                bemSeparator = _cachedProps3.bemSeparator,
                blockElements = _cachedProps3.blockElements,
                checked = _cachedProps3.checked,
                checkedPanels = _cachedProps3.checkedPanels,
                children = _cachedProps3.children,
                className = _cachedProps3.className,
                Component = _cachedProps3.component,
                contentId = _cachedProps3.contentId,
                disabled = _cachedProps3.disabled,
                disabledPanels = _cachedProps3.disabledPanels,
                hasContent = _cachedProps3.hasContent,
                id = _cachedProps3.id,
                index = _cachedProps3.index,
                modifiers = _cachedProps3.modifiers,
                onChangePanel = _cachedProps3.onChangePanel,
                name = _cachedProps3.name,
                style = _cachedProps3.style,
                type = _cachedProps3.type,
                value = _cachedProps3.value,
                visible = _cachedProps3.visible,
                props = _objectWithoutProperties(_cachedProps3, ['animateContent', 'bemModifiers', 'bemSeparator', 'blockElements', 'checked', 'checkedPanels', 'children', 'className', 'component', 'contentId', 'disabled', 'disabledPanels', 'hasContent', 'id', 'index', 'modifiers', 'onChangePanel', 'name', 'style', 'type', 'value', 'visible']);

            var panelBem = (0, _bem.bemClassName)(blockElements, 'panel', modifiers, bemSeparator);
            var ariaSelected = checked ? 'true' : 'false';

            return _react2.default.createElement(
                Component,
                _extends({}, props, {
                    'aria-expanded': hasContent && ariaSelected,
                    'aria-selected': ariaSelected,
                    className: !panelBem ? className : className ? panelBem + ' ' + className : panelBem,
                    role: 'tab',
                    style: _extends({}, style, { display: visible === 'hidden' ? 'none' : null })
                }),
                _react2.default.createElement('input', {
                    'aria-controls': contentId,
                    checked: checked,
                    'data-state': 'tabbordion',
                    disabled: disabled || visible === 'hidden',
                    id: id,
                    name: name,
                    onChange: this.onChange,
                    ref: this.getInputRef,
                    type: type,
                    value: value
                }),
                children
            );
        }
    }]);

    return TabPanel;
}(_react.PureComponent);

TabPanel.childContextTypes = {
    tabbordionPanel: _propTypes2.default.object
};

TabPanel.contextTypes = {
    bem: _propTypes2.default.object,
    tabbordion: _propTypes2.default.object
};

TabPanel.defaultProps = {
    component: 'li'
};

TabPanel.propTypes = {
    // state (these will override contextTypes if set)
    checked: _propTypes2.default.bool,
    className: _propTypes2.default.string,
    disabled: _propTypes2.default.bool,
    id: _propTypes2.default.string,
    index: _propTypes2.default.number,
    name: _propTypes2.default.string,
    type: _propTypes2.default.oneOf(['checkbox', 'radio']),
    value: _propTypes2.default.string,
    visible: _propTypes2.default.bool,
    // contents
    children: _propTypes2.default.node,
    component: _propTypes2.default.oneOfType([_propTypes2.default.func, _propTypes2.default.object, _propTypes2.default.string]),
    // BEM convention props
    modifiers: _propTypes2.default.array
};

exports.default = TabPanel;